#!/usr/bin/perl
use strict;

package MATT::FreeBSD;

=head1 NAME

	MATT::FreeBSD - FreeBSD scripting functions

=head1 SYNOPSIS

	FreeBSD.pm is a grouping of frequently used functions I've written
	for working with FreeBSD systems.

=head1 DESCRIPTION

	Usage examples for each subroutine:

=cut

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
require Exporter;

$VERSION = 0.4.0;

@ISA     = qw(Exporter);
@EXPORT  = qw(
	InstallPort
	InstallPackage
);
@EXPORT_OK = qw( CheckPortsAge );

use MATT::Utility;

sub InstallPort($$;$$)
{
	my ($name, $base, $booger, $check) = @_;
	my ($dir);

=head2 InstallPort

	use MATT::FreeBSD;
	InstallPort("net", "openldap2");

	That's it. Really. Honest. Nothing more. Well, OK, sometimes it can get a
	little more complex. InstallPort checks first to determine if a port is
	already installed and if so, skips right on by. It's very intelligent that
	way. However, sometimes port maintainers do goofy things and we need to
	override the directory directory we install from. A good example of this
	is currently openldap2. 

	If you want to install OpenLDAP 2, then you can install from any of:

		/usr/ports/net/openldap2
		/usr/ports/net/openldap20
		/usr/ports/net/openldap21
		/usr/ports/net/openldap22

	BTW: The second argument ("net") is what determines where in FreeBSD's ports
	tree the script can find OpenLDAP. If you pass along a third argument, we'll
	use it instead of the port name as the port directory to install from.

	On rare occasion, a port will get installed as a name other than the ports
	name. Of course, that wreaks all sorts of havoc so when one of them nasties
	is found, you can optionally pass along a fourth parameter which can be used
	as the port installation name to check with.

=cut

	if ($booger)
	{
		$dir = "/usr/ports/$base/$booger";
	} else {
		$dir = "/usr/ports/$base/$name";
	};

	CheckPortsAge("30");

	if (! $check) {
		$check = $name;
	};

	my $pkg_info = MATT::Utility::FindTheBin("pkg_info");
	my $grep     = MATT::Utility::FindTheBin("grep");

	my $r = `$pkg_info | cut -d" " -f1 | $grep $check-`; chomp $r;
	if ( ! $r ) {
		$r = `$pkg_info | cut -d" " -f1 | $grep $check`; chomp $r;
	};

	if ( $r ne "" )
	{
		printf "InstallPort: %-20s installed as (%s).\n", $name, $r;
		return 0;
	} else {
		print "InstallPort: installing $name...";
		chdir("$dir") or die "couldn't cd to $dir: $!\n";
		if ( $name eq "qmail" ) {
			SysCmd( "make enable-qmail clean");
			move("/usr/local/etc/rc.d/qmail.sh", "/usr/local/etc/rc.d/qmail.sh-dist") or die "$!";
		} elsif ( $name eq "ezmlm-idx" ) {
			SysCmd( "make -DWITH_MYSQL install");
			copy("work/ezmlm-0.53/ezmlmrc", "/usr/local/bin");
			SysCmd( "make clean");
		} elsif ( $name eq "mod_php4" ) {
			SysCmd( "make -DWITH_APACHE2 install");
			SysCmd( "make clean");
		} elsif ( $name eq "squirrelmail" ) {
			my $ans = YesOrNo("Are you using Apache2?");
			if ($ans) {
				SysCmd( "make -DWITH_APACHE2 install");
			} else {
				SysCmd( "make install");
			};
			SysCmd( "make clean");
		} else {
			SysCmd( "make install clean");
		};
		print "done.\n";
		return 1;
	};
};

sub CheckPortsAge($;$)
{
	my ($age, $url) = @_;

=head2 CheckPortsAge

	Checks how long it's been since you've updated your ports tree. Since
	the ports tree can be a roaming target, by making sure it's current
	before installing ports we can increase the liklihood of success. 

	use MATT::Freebsd;
	CheckPortsAge("30");

	That'll update the ports tree if it's been more than 30 days since
	it was last updated.

	You can optionally pass along a URL as the second argument from where
	to fetch the cvsup-ports file. If you have a custom one for your site,
	pass along the URL (minus the file name).

=cut

	if (!$url) { $url = "http://matt.simerson.net/computing/mail/toaster" };

	if ( -M "/usr/ports" > $age )
	{
		print "\n\nCheckPortsAge: CAUTION: You might want to update your ports tree!\n\n";

		if( YesOrNo( "\n\nWould you like me to do it for you?:") )
		{
			my $cvsupbin = MATT::Utility::FindTheBin("cvsup");

			if (!$cvsupbin) { InstallPackage("cvsup-without-gui"); };

			my $cvsup = "/etc/cvsup-ports";

			if ( !-e $cvsup )
			{
				FetchFile("$url/cvsup-ports");
				move("cvsup-ports", $cvsup);
			};
			SysCmd("$cvsupbin -g $cvsup");
		};
	}
	else
	{
		print "CheckPortsAge: Ports file is current (enough).\n";
	};
};

sub InstallPackage($;$$)
{   
	my ($package, $alt, $pkg_url) = @_;
	my $r;  

=head2 InstallPackage

	use MATT::FreeBSD;
	InstallPackage("ispell");

	That's all. If the package gets registered in FreeBSD's package
	registry and you want to check against that name (so it doesn't
	try installing a package that's already installed) instead, pass
	it along as the second argument.

	If you want to retrieve packages from a package site other than
	FreeBSD's (the default), then pass along that URL as the third
	argument. See the pkg_add man page for more details.

=cut

	my $pkg_info = MATT::Utility::FindTheBin("pkg_info");
	my $pkg_add  = MATT::Utility::FindTheBin("pkg_add");
	my $grep     = MATT::Utility::FindTheBin("grep");

	if ( $alt )
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $alt-`;
		if ( ! $r ) {
			$r = `$pkg_info | cut -d" " -f1 | $grep $alt`;
		};
	} else {
		$r = `$pkg_info | cut -d" " -f1 | $grep $package-`;
		if ( ! $r ) {
			$r = `$pkg_info | cut -d" " -f1 | $grep $package`;
		};
	};
	chomp $r;

	if ( $r )
	{
		printf "InstallPackage: %-20s installed as (%s).\n", $package, $r;
	}
	else
	{
		print "InstallPackage: installing $package....";
		$ENV{"PACKAGESITE"} = $pkg_url if $pkg_url;
		SysCmd("$pkg_add -r $package");
		print "done.\n";
	};
};

1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

	Needs more documentation.

=head1 SEE ALSO

	http://matt.simerson.net/computing/

=head1 COPYRIGHT

	Copyright 2003, Matt Simerson. All Right Reserved.

=cut
