#!/usr/bin/perl
package MATT::Mysql;

=head1 NAME

	MATT::Mysql - Common Mysql functions

=head1 SYNOPSIS

	MATT::Mysql is a grouping of frequently used functions I've written
	for Mysql.

=head1 DESCRIPTION

	I find myself using MySQL for a lot of things. Geographically distributed
	dns systems (MySQL replication), mail servers, and all the other fun stuff
	you'd use a RDBMS for. As such, I've got a growing pile of scripts that
	have lots of duplicated code in them. As such, the need for this Perl
	module grew.

=cut

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
	MysqlConnect 
	MysqlQuery 
	ParseDotFile 
	SetAutocommit 
	MysqlDBVars
);
@EXPORT_OK = qw( InstallMysqlTool );
$VERSION  = '0.4.0';

##
# Scripts that use this library (used in)
##
# mysql_replicate_manager v1.5+
# uron.net user_*.pl
# polls.pl
# nt_export_djb_update.pl
# toaster_setup.pl 

use Getopt::Std;
use POSIX qw(uname);

BEGIN {
	eval { require DBI };
	if ($@) {
		print "\nERROR: DBI.pm is not installed. Try this:\n\n";

		if ( (uname)[0] eq "FreeBSD" ) 
		{
			print "cd /usr/ports/databases/p5-DBI
make install clean\n\n\t - or -\n";
		};
		die "perl -MCPAN -e 'install DBI'\n\n";
	} else {
		DBI->import();
	};

	eval { require DBD::mysql };
	if ($@) 
	{
		print "\nERROR: DBD::mysql is not installed. Try this:\n\n";
		if ( (uname)[0] eq "FreeBSD" ) 
		{
			print "cd /usr/ports/databases/p5-DBD-mysql
make install clean\n\n\t - or -\n";
		};
		die "perl -MCPAN -e 'install DBD::mysql'\n\n";
	} else {
		DBD::mysql->import();
	};
};

sub MysqlQuery($$;$)
{
	my ($dbh, $query, $warn) = @_;

=head3 MysqlQuery

	MysqlQuery ($dbh, $sql, $warn)

	$dbh is the database handle you've already acquired.

	$sql is the SQL statement to execute.

	If $warn is set, we don't die if the query fails. This way you can
	decide when you call the sub whether you want it to die or return
	a failed $sth (and likely an error message).

	execute performs anything necessary to execute a prepared statement
	Always returns true regardless of # of rows affected.
	For non-Select, returns # of rows affected: No rows = 0E0
	For Select, simply starts query. Follow with fetch_*

=cut

	if ($warn) {
		if ( my $sth = $dbh->prepare($query) ) {
			$sth->execute or warn "couldn't execute: $sth->errstr\n";
			#$dbh->commit  or warn "couldn't commit: $sth->errstr\n";
			return $sth;
		} else {
			warn "couldn't prepare: $DBI::errstr\n";
			return $sth;
		};
	} else {
		if ( my $sth = $dbh->prepare($query) ) {
			$sth->execute or die "couldn't execute: $sth->errstr\n";
			#$dbh->commit  or die "couldn't commit: $sth->errstr\n";
			return $sth;
		} else {
			die "couldn't prepare: $DBI::errstr\n";
		};
	};
};

sub MysqlConnect($;$$)
{
   my ( $dot, $warn, $debug) = @_;
	my $dbh;

=head3 MysqlConnect

	MysqlConnect($dot, $warn, $debug);

	$dot is a hashref of key/value pairs in the same format you'd find in
	~/.my.cnf. No coincidentally, that's where it expects you'll be
	getting them from.

	$warn allows you to determine whether to die or warn on failure or
	error. To warn, set $warn to a non-zero value. 

	$debug will print out helpful debugging messages should you be having
	problems.

=cut

	my $ac  = SetAutocommit( $dot );
	my $dbv = MysqlDBVars  ( $dot );
	my $dsn = "DBI:$dbv->{'driver'}:database=$dbv->{'db'};host=$dbv->{'host'};port=$dbv->{'port'}";

	if ($warn) 
	{
		$dbh = DBI->connect($dsn, $dbv->{'user'}, $dbv->{'pass'},
			{ RaiseError => 0, AutoCommit => $ac } )
				or warn "db connect failed: $!\n";
	} 
	else 
	{
		$dbh = DBI->connect($dsn, $dbv->{'user'}, $dbv->{'pass'},
			{ RaiseError => 0, AutoCommit => $ac } ) 
				or die "db connect failed: $!\n";
	};
	my $drh = DBI->install_driver($dbv->{'driver'});

	return ($dbh, $dsn, $drh);
};

sub MysqlDBVars($)
{
	my ($val) = @_;
	my ($driver, $db, $host, $port, $user, $pass, $dir);

=head3 MysqlDBVars

	This sub is called internally by MysqlConnect and is used principally 
	to set some reasonable defaults should you not pass along enough 
	connection parameters in $dot. 

=cut

	if ( $val->{'driver'} && $val->{'driver'} ne "" ) { 
		$driver= $val->{'driver'} } else { $driver= "mysql" };

	if ( $val->{'db'}     && $val->{'db'}     ne "" ) {
		$db    = $val->{'db'}     } else { $db    = "mysql" };

	if ( $val->{'host'}   && $val->{'host'}   ne "" ) {
		$host  = $val->{'host'}   } else { $host  = "localhost" };

	if ( $val->{'port'}   && $val->{'port'}   ne "" ) {
		$port  = $val->{'port'}   } else { $port  = "3306" };

	if ( $val->{'user'}   && $val->{'user'}   ne "" ) {
		$user  = $val->{'user'}   } else { $user  = "root" };

	if ( $val->{'pass'}   && $val->{'pass'}   ne "" ) {
		$pass  = $val->{'pass'}   } else { $pass  = "" };

	if ( $val->{'dir_m'}  && $val->{'dir_m'}  ne "" ) {
		$dir ="/var/db/mysql"} else { $dir   = $val->{'dir_m'} };     
  
	my %master = ( driver  => $driver,     db      => $db,      host    => $host,
						port    => $port,       user    => $user,    pass    => $pass,
						dir     => $dir );
	return \%master;
};

sub SetAutocommit {
	my ($dot) = @_;

#	SetAutocommit

	if ($dot->{'autocommit'} && $dot->{'autocommit'} ne "" ) { 
		return $dot->{'autocommit'};
	} else {
		#  Default to autocommit.
		return 1;
	};
};

sub ParseDotFile($$;$)
{
	my ($file, $start, $debug) = @_;
   my ($homedir)                = (getpwuid ($<))[7];
   my $dotfile                  = "$homedir/$file";

=head3 ParseDotFile

	ParseDotFile ($file, $start, $debug)

	$file is the file to be parsed.

	$start is the [identifier] where we begin looking for settings. 
	This expects the format used in .my.cnf MySQL configuration files.

=cut

   if (-r $dotfile) {
      my (%array);
      my $gotit = 0;

      print "ParseDotFile: $dotfile\n" if ($debug);
      open(DOT, $dotfile) or warn "WARNING: Can't open $dotfile: $!";
         while ( <DOT> ) {
            next if /^#/;
            my $line = $_; chomp $line;
            if ( !$gotit ) {
               print "1. $line\n" if $debug;
               if ($line eq $start) {
                  $gotit = 1; 
						next;
               };
            } else {
               if ( $line =~ /^\[/ ) { last };
               print "2. $line\n" if $debug;
               $line =~ /(\w+)\s*=\s*(.*)\s*$/;
               $array{$1} = $2 if $1;
            };
         };

         if ($debug) {
            foreach my $key ( keys %array ) {
               print "hash: $key\t=$array{$key}\n";
            };
         };
      close(DOT);
      return \%array;
   } else {
		warn "WARNING: ParseDotFile: can't read $dotfile!\n";
		return 0;
	};
};

sub InstallMysqlTool($;$$)
{
	my ($package, $httpdir, $confdir) = @_;

	my $site = "http://www.dajoba.com/projects/mysqltool";
	$httpdir = "/usr/local/www" if (!$httpdir);
	$confdir = "/usr/local/etc" if (!$confdir);

	if ( (uname)[0] eq "FreeBSD" ) 
	{
		use MATT::FreeBSD;
		InstallPort("p5-Crypt-Blowfish", "security");
		InstallPort("p5-DBI",            "databases");
		InstallPort("p5-Apache-DBI",     "www");
		InstallPort("p5-DBD-mysql",      "databases");
		InstallPort("p5-SOAP-Lite",      "net");
	} else {
		use CPAN;
		install "Crypt::Blowfish";
		install "DBI";
		install "Apache::DBI";
		install "DBD::mysql";
		install "SOAP::Lite";
	};

	CdSrcDir("/usr/local/www");
	
	if (!-e "$package.tar.gz" ) 
	{
		FetchFile("$site/$package.tar.gz");
	};

	if ( -d "$package" )
	{
		my $r = SourceWarning($package, 1);
		if (! $r) { die "sorry, I can't continue.\n"; };
	};

	my $tar = FindTheBin("tar");
	SysCmd("$tar -xzf $package.tar.gz");
	chdir($package);
	SysCmd( "perl Makefile.PL");
	SysCmd( "make");
	SysCmd( "make install clean");

	if ( !-e "$confdir/apache/mysqltool.conf" ) 
	{
		move("htdocs/mysqltool.conf", "$confdir/apache") or die "move failed: $!\n";
	};

	if ( !-e "$httpdir/data/mysqltool" ) 
	{
		FetchFile("http://matt.simerson.net/computing/sql/mysqltool.index.txt");
		move("mysqltool.index.txt", "htdocs/index.cgi");
		move("htdocs", "$httpdir/data/mysqltool") or die "move failed: $!\n";
	};
};



1;
__END__

=head2 Dependencies

	DBI.pm     - /usr/ports/databases/p5-DBI
	DBD::mysql - /usr/ports/databases/p5-DBD-mysql

	In order to use this module, you must have DBI.pm and DBD::Mysql
	installed. If they are not installed and you attempt to use this
	module, you should get some helpful error messages telling you how
	to install them.

=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

	Don't export any of the symbols by default. Move all symbols
	to EXPORT_OK and explicitely pull in the required ones in 
	programs that need them.

=head1 SEE ALSO

	http://matt.simerson.net/computing/sql/

=head1 COPYRIGHT

	Copyright 2003, Matt Simerson. All Right Reserved.

=cut
